/*   
   creation of model files for 1D models from unitcells (see below)
     
     (c) JS 2001-2010
     
    v0.5 cosmetic + bond vector R
    v0.4 make D=1 lattices from one unitcell
       
    example:
       
     5  ------0---1----5---6----
     4         \ /      \ /
     3          2        7
     2         / \      / \    
     1  ------3---4----8---9----
     0   
             0123456
             
  v0.3 - J1J2-chain added, p_t set
                       
*/
#include <stdlib.h>
#include <stdio.h>

#undef PChain
#define J1J2Chain
#undef KagomeChain
#undef TriangleChain
#undef DiamondChain

/* S=1-S=1/2-P-chain
 *          
 *   3      1---3---1--
 *   2      | X   X | X
 *   1      0---2---0--
 *   0123
 *  
 */
#ifdef PChain
#define NUMsites 4
#define NUMbonds 9 
#define NUMidx   2 
int evec[2]={3,99};
int sites[NUMsites][2]={{1,1},{1,2},{2,1},{2,2}};  /* 4 sites */
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,2,1,0}, {0,3,1,0}, {1,2,1,0}, {1,3,1,0},
 {2,0,1,1}, {2,1,1,1}, {3,0,1,1}, {3,1,1,1},
 {0,1,2,0}};
#endif


/* J2=2J1
 *   0===0===0===0=== J2
 *    \ / \ / \ / \ / J1
 *   ==0===0===0===0= J2
 * N=8 nud=4,4 sym_ud=+1 sym_k=-2 0 0
 *  q_0_0+ n1=7  -3.65109341  -1.80193774  -0.72610945  -0.44504187
 *  q_0_1+ n1=0
 *  q_4_0+ n1=5  -2.69962815  -0.76087672   0.00000000   1.00000000   1.46050487
 *  q_4_1+ n1=1   0.00000000
 *  q_0_0- n1=1   0.00000000
 *  q_0_1- n1=2  -1.00000000   0.00000000
 *  q_4_0- n1=0
 *  q_4_1- n1=4  -3.12841906  -1.20163968   0.00000000   1.33005874
 */
#ifdef J1J2Chain
#define NUMsites 1
#define NUMbonds 2
#define NUMidx   2
int evec[2]={1,99};
int sites[NUMsites][2]={{1,1}};
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,0,1,1},  {0,0,2,2}};
#endif

/* J2=(3/2)J1 */
#ifdef KagomeChain
#define NUMsites 5
#define NUMbonds 8
#define NUMidx   2
int evec[2]={4,99};
int sites[NUMsites][2]={{1,3},{3,3},{2,2},{1,1},{3,1}};
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,1,1,0},  {3,4,1,0}, {0,2,1,0},  {2,4,1,0},
 {1,2,1,0},  {2,3,1,0}, {1,0,2,1},  {4,3,2,1}};
#endif

/* J2=2J1 */
#ifdef TriangleChain
#define NUMsites 2
#define NUMbonds 3
#define NUMidx   2
int evec[2]={2,99};
int sites[NUMsites][2]={{1,1},{2,2}};
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,1,2,0},  {1,0,2,1}, {0,0,1,1}};
#endif

#ifdef DiamondChain
#define NUMsites 3
#define NUMbonds 6
#define NUMidx   1
int evec[2]={2,99};
int sites[NUMsites][2]={{1,1},{1,3},{2,2}};
int bonds[NUMbonds][4]={ /* site1 site2 idx-J cell-x */
 {0,2,1,0},  {1,2,1,0}, {0,0,1,1},
 {1,1,1,1},  {2,0,1,1}, {2,1,1,1}};
#endif

int initvars(int n0,int pbc){
   int i,j,nn,nw;
   nn=n0*NUMsites;
   nw=n0*NUMbonds;
   printf("# def-file, 1D-chain\n"
	"# automaticly generated by m_1D v0.5 n0=%d PBC\n"
	"#===edge=== %d 0 0 999\n"
	"#+++model+++\n"
	"# NN=Number_of_sites Nw=Number_of_bonds [Nq=quad_terms]\n"
	"%d %d 0\n"
	"# s=site  XYZ=position #=index\n"
	"# s Rx Ry Rz [e#] [U#]\n",
     n0,n0*evec[0],nn,nw);
   for(i=0;i<n0;i++)for(j=0;j<NUMsites;j++)
     printf(" %2d %2d %2d  0 e8 u7\n",
        i*NUMsites+j, sites[j][0]+i*evec[0], sites[j][1]);
   /* n=index s1,s2=sites R=PBCvector_to_UC t,j,v,w=parameters */
   printf("# Bond s1 s2 [R= # # #] [t#] [j#] [v#] [w#]\n");
   for(i=0;i<n0;i++)for(j=0;j<NUMbonds;j++)
     printf(" %2d %2d %2d  R= 1 0 0  t%d j%d v0 w%d %c\n",
      i*NUMbonds+j,            /* bond or pair index */
      bonds[j][0]+NUMsites*i,  /* site 1 of the pair */
      bonds[j][1]+NUMsites*((i+bonds[j][3]+n0)%n0), /* site 2 of the pair */
      bonds[j][2]+NUMidx,  /* t */
      bonds[j][2],         /* J */
      bonds[j][2]+2*NUMidx+1, /* w */ 
      (((i+bonds[j][3]>=n0 || i+bonds[j][3]<0) && pbc==1)?'+':' ') );
   printf("#---model---\n");
   return 0;
}


main(int argc,char *argv[]){
   if(argc!=2){
     fprintf(stderr,
       " generate daten.def for an 1D chain v0.5 JS2010\n"
       " use: m_1D ?length?\n"
       "   length: 1..6  chain length in unit cells\n");
     exit(1);
   }
   initvars(atoi(argv[1]),1);
}
